@@ -10,6 +10,7 @@ models.
 
 from __future__ import annotations
 
+from pathlib import Path
 from typing import Iterable, List, Optional, Sequence, Tuple, Type as TypingType
 
 
@@ -84,6 +85,7 @@ class TransferDescription(ContainerElement):
     def __init__(self) -> None:
         super().__init__(name=None)
         self._models: List[Model] = []
+        self._primary_source: Optional[Path] = None
 
     def add_model(self, model: "Model") -> None:
         self._models.append(model)
@@ -92,6 +94,32 @@ class TransferDescription(ContainerElement):
     def getModels(self) -> Sequence["Model"]:
         return tuple(self._models)
 
+    def setPrimarySource(self, source: Optional[Path]) -> None:
+        self._primary_source = source
+
+    def getModelsFromLastFile(self) -> Sequence["Model"]:
+        if not self._models:
+            return ()
+
+        last_model = self._models[-1]
+        source = getattr(last_model, "_source", None)
+        target_sources: List[Path] = []
+        if self._primary_source is not None:
+            target_sources.append(self._primary_source)
+        if isinstance(source, Path) and source not in target_sources:
+            target_sources.append(source)
+
+        for candidate in target_sources:
+            matches = [
+                model
+                for model in self._models
+                if getattr(model, "_source", None) == candidate
+            ]
+            if matches:
+                return tuple(matches)
+
+        return (last_model,)
+
     def find_model(self, name: str) -> Optional["Model"]:
         for model in self._models:
             if model.getName() == name:
@@ -173,6 +201,8 @@ class Topic(ContainerElement):
         self._classes: List[Table] = []
         self._structures: List[Table] = []
         self._associations: List["Association"] = []
+        self._oid_type: Optional[Type] = None
+        self._basket_oid_type: Optional[Type] = None
 
     def add_class(self, table: "Table") -> "Table":
         self._classes.append(table)
@@ -195,6 +225,22 @@ class Topic(ContainerElement):
     def getAssociations(self) -> Sequence["Association"]:
         return tuple(self._associations)
 
+    def setOIDType(self, oid_type: Optional[Type]) -> None:
+        self._oid_type = oid_type
+        if oid_type is not None:
+            self._register_child(oid_type)
+
+    def getOIDType(self) -> Optional[Type]:
+        return self._oid_type
+
+    def setBasketOIDType(self, oid_type: Optional[Type]) -> None:
+        self._basket_oid_type = oid_type
+        if oid_type is not None:
+            self._register_child(oid_type)
+
+    def getBasketOIDType(self) -> Optional[Type]:
+        return self._basket_oid_type
+
 
 class Type(Element):
     """Representation of a type reference or built-in type."""
@@ -202,6 +248,151 @@ class Type(Element):
     def __init__(self, name: Optional[str]) -> None:
         super().__init__(name=name)
 
+    # ------------------------------------------------------------------
+    def getDisplayName(self) -> str:
+        name = self.getName()
+        return name if name else "Unknown"
+
+
+class TextType(Type):
+    def __init__(
+        self,
+        *,
+        kind: str,
+        max_length: Optional[int] = None,
+        normalized: bool = True,
+        name: Optional[str] = None,
+    ) -> None:
+        super().__init__(name=name or kind)
+        self._kind = kind
+        self._max_length = max_length
+        self._normalized = normalized
+
+    def getKind(self) -> str:
+        return self._kind
+
+    def getMaxLength(self) -> Optional[int]:
+        return self._max_length
+
+    def isNormalized(self) -> bool:
+        return self._normalized
+
+    def getDisplayName(self) -> str:
+        label = self._kind
+        if self._kind.upper() in {"TEXT", "MTEXT"} and self._max_length:
+            label = f"{label}*{self._max_length}"
+        return label
+
+
+class NumericType(Type):
+    def __init__(
+        self,
+        *,
+        minimum: Optional[str] = None,
+        maximum: Optional[str] = None,
+        name: Optional[str] = None,
+    ) -> None:
+        super().__init__(name=name)
+        self._minimum = minimum
+        self._maximum = maximum
+
+    def getMinimum(self) -> Optional[str]:
+        return self._minimum
+
+    def getMaximum(self) -> Optional[str]:
+        return self._maximum
+
+    def getDisplayName(self) -> str:
+        name = super().getName()
+        if name:
+            return name
+        if self._minimum is not None or self._maximum is not None:
+            left = self._minimum if self._minimum is not None else ""
+            right = self._maximum if self._maximum is not None else ""
+            if left and right:
+                return f"{left}..{right}"
+            return left or right or "NUMERIC"
+        return "NUMERIC"
+
+
+class BlackboxType(Type):
+    """Representation of ``BLACKBOX`` types."""
+
+    KIND_XML = "XML"
+    KIND_BINARY = "BINARY"
+
+    def __init__(self, *, kind: str, name: Optional[str] = None) -> None:
+        kind_value = kind.upper()
+        super().__init__(name=name or f"BLACKBOX {kind_value}".strip())
+        self._kind = kind_value
+
+    def getKind(self) -> str:
+        return self._kind
+
+    def getDisplayName(self) -> str:
+        if self._kind == self.KIND_BINARY:
+            return "Blackbox Binary"
+        if self._kind == self.KIND_XML:
+            return "Blackbox Xml"
+        label = self._kind.title() if self._kind else ""
+        return f"Blackbox {label}".strip()
+
+
+class GeometryType(Type):
+    def __init__(self, *, kind: str, definition: str, name: Optional[str] = None) -> None:
+        super().__init__(name=name or kind.upper())
+        self._kind = kind
+        self._definition = definition
+
+    def getKind(self) -> str:
+        return self._kind
+
+    def getDefinition(self) -> str:
+        return self._definition
+
+    def getDisplayName(self) -> str:
+        return self._kind
+
+
+class AreaType(GeometryType):
+    def __init__(self, *, definition: str, name: Optional[str] = None) -> None:
+        super().__init__(kind="Area", definition=definition, name=name)
+
+
+class SurfaceType(GeometryType):
+    def __init__(self, *, definition: str, name: Optional[str] = None) -> None:
+        super().__init__(kind="Surface", definition=definition, name=name)
+
+
+class MultiSurfaceType(GeometryType):
+    def __init__(self, *, definition: str, name: Optional[str] = None) -> None:
+        super().__init__(kind="MultiSurface", definition=definition, name=name)
+
+
+class MultiAreaType(GeometryType):
+    def __init__(self, *, definition: str, name: Optional[str] = None) -> None:
+        super().__init__(kind="MultiArea", definition=definition, name=name)
+
+
+class PolylineType(GeometryType):
+    def __init__(self, *, definition: str, name: Optional[str] = None) -> None:
+        super().__init__(kind="Polyline", definition=definition, name=name)
+
+
+class MultiPolylineType(GeometryType):
+    def __init__(self, *, definition: str, name: Optional[str] = None) -> None:
+        super().__init__(kind="MultiPolyline", definition=definition, name=name)
+
+
+class CoordType(GeometryType):
+    def __init__(self, *, definition: str, name: Optional[str] = None) -> None:
+        super().__init__(kind="Coord", definition=definition, name=name)
+
+
+class MultiCoordType(GeometryType):
+    def __init__(self, *, definition: str, name: Optional[str] = None) -> None:
+        super().__init__(kind="MultiCoord", definition=definition, name=name)
+
 
 class EnumerationType(Type):
     def __init__(self, name: Optional[str], literals: Sequence[str]) -> None:
@@ -211,6 +402,109 @@ class EnumerationType(Type):
     def getLiterals(self) -> Sequence[str]:
         return tuple(self._literals)
 
+    def getDisplayName(self) -> str:
+        name = self.getName()
+        if name:
+            return name.split(".")[-1]
+        return ", ".join(self._literals) if self._literals else "Enumeration"
+
+
+class EnumTreeValueType(Type):
+    def __init__(self, base_domain: str, name: Optional[str] = None) -> None:
+        super().__init__(name=name)
+        self._base_domain = base_domain
+
+    def getBaseDomain(self) -> str:
+        return self._base_domain
+
+    def getDisplayName(self) -> str:
+        return self._base_domain.split(".")[-1] if self._base_domain else "Enumeration"
+
+
+class FormattedType(Type):
+    def __init__(
+        self,
+        *,
+        base_domain: str,
+        picture: Optional[str] = None,
+        minimum: Optional[str] = None,
+        maximum: Optional[str] = None,
+        name: Optional[str] = None,
+    ) -> None:
+        super().__init__(name=name)
+        self._base_domain = base_domain
+        self._picture = picture
+        self._minimum = minimum
+        self._maximum = maximum
+
+    def getBaseDomain(self) -> str:
+        return self._base_domain
+
+    def getPicture(self) -> Optional[str]:
+        return self._picture
+
+    def getMinimum(self) -> Optional[str]:
+        return self._minimum
+
+    def getMaximum(self) -> Optional[str]:
+        return self._maximum
+
+    def getDisplayName(self) -> str:
+        base = self._base_domain or self.getName()
+        if not base:
+            return "FORMAT"
+        return base.split(".")[-1]
+
+
+class TextOIDType(Type):
+    def __init__(self, oid_type: Type, name: Optional[str] = None) -> None:
+        super().__init__(name=name)
+        self._oid_type = oid_type
+        self._register_child(oid_type)
+
+    def getOIDType(self) -> Type:
+        return self._oid_type
+
+    def getDisplayName(self) -> str:
+        return f"OID {self._oid_type.getDisplayName()}".strip()
+
+
+class TypeAlias(Type):
+    def __init__(self, target: str) -> None:
+        super().__init__(name=target)
+        self._target = target
+
+    def getAliasing(self) -> str:
+        return self._target
+
+    def getDisplayName(self) -> str:
+        return self._target.split(".")[-1] if self._target else "Unknown"
+
+
+class ReferenceType(Type):
+    def __init__(self, target: str, name: Optional[str] = None) -> None:
+        super().__init__(name=name)
+        self._target = target
+
+    def getReferred(self) -> str:
+        return self._target
+
+    def getDisplayName(self) -> str:
+        target = self._target.split(".")[-1] if self._target else "Unknown"
+        return target
+
+
+class ObjectType(Type):
+    def __init__(self, target: str, name: Optional[str] = None) -> None:
+        super().__init__(name=name or target)
+        self._target = target
+
+    def getTarget(self) -> str:
+        return self._target
+
+    def getDisplayName(self) -> str:
+        return self._target.split(".")[-1] if self._target else "Unknown"
+
 
 class Cardinality:
     def __init__(self, minimum: int, maximum: int) -> None:
@@ -372,6 +666,7 @@ class Table(Viewable):
         super().__init__(name=name, abstract=abstract)
         self._kind = kind
         self._identifiable = identifiable
+        self._oid_type: Optional[Type] = None
 
     def isIdentifiable(self) -> bool:
         return self._identifiable
@@ -379,6 +674,14 @@ class Table(Viewable):
     def getKind(self) -> str:
         return self._kind
 
+    def setOIDType(self, oid_type: Optional[Type]) -> None:
+        self._oid_type = oid_type
+        if oid_type is not None:
+            self._register_child(oid_type)
+
+    def getOIDType(self) -> Optional[Type]:
+        return self._oid_type
+
 
 class Association(ContainerElement):
     def __init__(self, name: Optional[str]) -> None:
